// ===============================================================================================
package gamelib;

import javax.swing.JOptionPane;
import java.util.Calendar;
import java.io.*;
// -----------------------------------------------------------------------------

/**
Diese Klasse dient dem Lesen und Schreiben von Highscorelisten.
In dieser Version werden 10 Records erzeugt, die den Namen, die erzielte Punktzahl und das Datum des Eintrags enthalten. Ein Name kann bis zu 24 Zeichen lang sein. Das Datum liegt im NATO-Format vor.
Die Eingabe erfolgt durch einen Swing-Requester, der seine Text-Strings im Konstruktor uebergeben bekommt.
Zum Einsortieren wird der Bubblesort-Algorithmus verwendet.
Diese Klasse braucht die Klasse loadsave
@author uja
@version 20071223
@see loadsave
*/
public class hiscore
{
  final int himax=10;
  final int snamlen=24;

  String hscfile,geschafft,bedauern;
  int punkte[]  =new int[himax+1];
  String sname[]=new String[himax+1];
  String datum[]=new String[himax+1];

  loadsave fh=new loadsave();

// Konstruktor und Methoden: ---------------------------------------------------------------------------------
/** Generieren einer Highscoreliste in der Datei outfil.
@param outfil Datei mit der Highscore-Liste, wird bei Bedarf angelegt
@param ges Text, der bei Eintrag auf dem Requester erscheinen soll
@param nichtges Text, der bei Nichterreichen der Mindestpunktzahl erscheinen soll.
*/
  public hiscore(String outfil,String ges,String nichtges)
  { hscfile=outfil;
    load_hsc();
    geschafft=ges;
    bedauern=nichtges;
  }

/** Anzahl der maximalen Records in der Highscoreliste.
@return Anzahl Records in Highscoreliste
 */
  public int get_maxeintrag()         { return himax;  }

/** Spielername auf einem bestimmten Platz.
@param platz Record-Id des gesuchten Eintrags
@return Name des Spielers
 */
  public String get_snamen(int platz) { return sname[platz];  }

/** Punktestand auf einem bestimmten Platz.
@param platz Record-Id des gesuchten Eintrags
@return Punktestand des gesuchten Eintrages
*/
  public int get_punkte(int platz)    { return punkte[platz]; }

/** Datum des erreichten Punktestandes auf einem bestimmten Platz.
@param platz Record-Id des gesuchten Eintrags
@return Datum (NATO-Format) des gesuchten Eintrages
*/
  public String get_datum(int platz)  { return datum[platz];  }

/** Zuruecksetzen aller Records auf den Punktestand 0 und das aktuelle Datum.*/
  public void reset_hsc()
  { String out="",dat=get_datum();
    for (int i=0; i<himax; i++) out=out+"0;"+dat+";...\n";
    fh.savefile(hscfile,out);
    load_hsc();
  }

/** Eingabe des Spielernamens oder Nachricht, dass es nicht gereicht hat.
Die Methode verwendet einen Requester. Der Requester kann mit Cancel abgebrochen werden. Die Highscoreliste wird bei Bedarf neu gespeichert und wieder eingelesen.
@param pkt erreichte Punktzahl
@see javax.swing.JPanel
*/
  public void eintrag(int pkt)
  { load_hsc(); // safety first! Hier darf kein Puffer stoeren!
    if (pkt>punkte[himax-1])
    { sname[himax]=JOptionPane.showInputDialog(""+pkt+geschafft); 
      if (sname[himax]!=null) if (sname[himax].length()>0)
      { if (sname[himax].length()>snamlen) sname[himax]=sname[himax].substring(0,snamlen);
        punkte[himax]=pkt;
        datum[himax]=get_datum();
        bubblesort();
        save_hsc();
      }
    }
    else JOptionPane.showMessageDialog(null,""+pkt+bedauern,"Info:",JOptionPane.INFORMATION_MESSAGE);
    load_hsc();
  }

// ===========================================================================================================

 void load_hsc()
 { String rec[]=fh.get_filelines(hscfile);
   String temp[]=new String[3];
   for (int i=0; i<rec.length; i++)
   { temp=rec[i].split(";");
     punkte[i]=Integer.parseInt(temp[0]);
     datum[i]=temp[1];
     sname[i]=temp[2];
   }
 }

  void save_hsc()
  { String out="";
    bubblesort();
    for (int i=0; i<himax; i++) out=out+punkte[i]+";"+datum[i]+";"+sname[i]+"\n";
    fh.savefile(hscfile,out); 
  }

  void bubblesort()
  { int i,j,k;
    String s;
    boolean tau;
    for (i=0; i<himax; i++) do
    { tau=false;
      for (j=i+1; j<=himax; j++) if (punkte[i]<punkte[j])
      { tau=true;
        k=punkte[i]; punkte[i]=punkte[j]; punkte[j]=k;
        s=datum[i];  datum[i]=datum[j];   datum[j]=s;
        s=sname[i];  sname[i]=sname[j];   sname[j]=s;
      }
    }
    while (tau);
  }

  String get_datum()
  { Calendar cal=Calendar.getInstance();
    String t="";
    int t1=cal.get(Calendar.DATE);    if (t1<10) t=t+"0"; t=t+t1+".";
    t1=cal.get(Calendar.MONTH)+1;     if (t1<10) t=t+"0"; t=t+t1+".";
    t1=cal.get(Calendar.YEAR)%100;    if (t1<10) t=t+"0"; t=t+t1+" ";
    t1=cal.get(Calendar.HOUR_OF_DAY); if (t1<10) t=t+"0"; t=t+t1+":";
    t1=cal.get(Calendar.MINUTE);      if (t1<10) t=t+"0"; t=t+t1+":";
    t1=cal.get(Calendar.SECOND);      if (t1<10) t=t+"0"; t=t+t1;
    return t;
  }

}
