// ===============================================================================================
package gamelib;

import java.io.*;

/**
Diese Klasse dient dem Lesen und Schreiben Dateien und Ressourcen.
Die Klasse unterscheidet zwischen Dateien und Ressourcen. Dateien koennen lesen und geschrieben werden, Ressourcen sind aus Sicherheitsgruenden meist read-only. Ein typischer Vertreter der Ressourcen sind Levelfiles und Hilfetexte, die ueblicherweise im JAR-File untergebracht sind, waehrend fuer Spielspeicher und Optionslisten Dateien verwendet werden sollten, die zumeist im gleichen Directory liegen wie das JAR-File.
Die Klasse benutzt die Stream-Technik.
@author uja
@version 20071223
*/

public class loadsave
{
/** Erzeugen eines loadsave-Handles */
  public loadsave() { }

/** Inhalt einer Datei in einem einzigen String. 
@param fil Pfad/Dateiname
@return Dateiinhalt als einziger String
*/
  public String get_fileinhalt(String fil)  { return loadfile(fil); }

/** Inhalt einer Datei zeilenweise in Strings verpackt. 
@param fil Pfad/Dateiname
@return Dateiinhalt zeilenweise im String-Vektor
*/
  public String[] get_filelines(String fil) { return loadfile(fil).split("\n"); }

/** Inhalt einer Ressource in einem einzigen String. 
@param fil Pfad/Dateiname
@return Ressourceninhalt als einziger String
*/
  public String get_ressourceinhalt(String fil) { return loadressource(fil); }

/** Inhalt einer Ressource zeilenweise in Strings verpackt. 
@param fil Pfad/Dateiname
@return Ressourceninhalt zeilenweise im String-Vektor
*/
  public String[] get_ressourcelines(String fil) { return loadressource(fil).split("\n"); }

/** Schreiben eines Strings in eine Datei.
Die Datei wird angelegt, falls sie noch nicht existiert, sonst ueberschreiben. Auf passende Berechtigugngen ist zu achten.
@param fil Pfad/Dateiname
@param inhalt Dateiinhalt als einziger String
*/
  public void set_fileinhalt(String fil,String inhalt) { savefile(fil,inhalt); }

/** Zeilenweises Schreiben eines String-Vektors in eine Datei.
Die Datei wird angelegt, falls sie noch nicht existiert, sonst ueberschreiben. Auf passende Berechtigugngen ist zu achten.
@param fil Pfad/Dateiname
@param inhalt Zeilen als String-Vektor
*/
  public void set_filelines(String fil,String[]inhalt)
  { String t="";
    for (int i=0; i<inhalt.length; i++) t=t+inhalt[i]+"\n";
    savefile(fil,t);
  }

// ===========================================================================================================
  String loadfile(String infile)
  { String s,t="";
    try
    { BufferedReader in=new BufferedReader(new InputStreamReader(new FileInputStream(infile), "UTF8"));
      do { s=in.readLine(); if (s!=null) t=t+s+"\n"; } while (s!=null);
      in.close();
    }
    catch (IOException e) { System.out.println(">>> loadsave: "+infile+" not found"); }
    return t;
  }

  String loadressource(String infile)
  { String s,t="";
    try
   { InputStream ips= getClass().getResourceAsStream(infile);
     BufferedReader in = new BufferedReader(new InputStreamReader(ips,"UTF8"));
     do
     { s=in.readLine();
       if (s!=null) t=t+s+"\n";
     }
     while (s!=null);
     in.close();
   }
   catch (NullPointerException e) { System.out.println(">>> loadsave: ressource "+infile+" not found"); }
   catch (IOException e) { System.out.println(">>> loadsave: ressource "+infile+" not found or has wrong format"); }
   return t;
 }

  static void savefile(String outfil,String inhalt)
  { try
    { BufferedWriter out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(outfil), "UTF8"));
      out.write(inhalt);
      out.close();
    }
    catch (UnsupportedEncodingException e) { System.out.println(">>> loadsave: no UTF-8 upport"); }
    catch (IOException e) { System.out.println(">>> loadsave: Could not write "+outfil); }
  }

}
