package schlangengrube;
import gamelib.sprite;
import gamelib.sound;

/**
Threadgesteuertes Arcade-Spiel mit Highscoreliste, Pauenfunktion und Levelspeicher.
Die Framerate wird aus der Klasse spielfeld uebernommen.
Die aufrufende Klasse main wird als parent uebergeben: main beherbergt das Sprachmodul und andere Listen
und regelt die Kommunikation mit anderen Programmteilen.
@author uja
@version 20071224
@see gamelib.sprite
@see spielfeld
*/

class spiel implements Runnable
{
  static final int maxsprites=64,ktic=24; // maximale Anzahl Kugeln, Abstand der Kugeln in Kette 
  static final int maxhaufen=8,maxfloater=8,maxstep=3000; // Levelparameter
  static final int joker=6,sonder=7,explo=8,weg=10;       // Sondersprites und Effekte
  static final int shooterline=320;                       // Basis, von der aus geschossen wird

  Thread mainloop;

  // Level=Spiellevel, punkte=Punktestend, punkte0=Punkte zu Beginn des Levels
  // numfarben=Anzahl der Farben im aktuellen Level, maxcount=Anzahl der Kugeln im Level
  // stopcount=Anzahl der Tics, die die Schlange bei Paralysieren aussetzen soll
  // weitermit=Steuerung Thread-Ablauf, multi=Punkte-Multiplikator bei rekursivem Triplett
  // sxmax=Pointer auf Screenbreite fuer Shooterbeweglichkeit
  static int numfarben=3,anzahl=32,maxcount=32,stopcount=0;    // Levelparameter
  static int level,punkte=0,punkte0=0,multi=0,weitermit=0;               // Bewertungen
  static boolean gover,pausiert,soundon,level_fertig,joker_on,sonder_on; // ein paar Flags

  // bewegliche Teile:
  static sprite start,ziel,flyer,shooter;
  static sprite[] kugel=new sprite[maxsprites];
  static sprite[] haufen=new sprite[maxhaufen];   // Kugelbremsen=Schildkroeten
  static sprite[] floater=new sprite[maxfloater]; // herumfliegenden Goodies, ab V.1.1

  // Listen:
  static int[] kliste=new int[maxsprites+1]; // Kugelkettenliste, wird in "neu" angelegt
  static int[] dliste=new int[maxsprites+1]; // zu droppende Kugelkette fuer Effektdarstellungen
  static int[][] koo =new int[maxstep][2];   // Koordinatenliste der Ketten-Flugphasen, erweitert auf 1500 ab V.1.1

  static String messi="";

  // Pointer:
  main mama;
  sound zaund;
  static int sxmax,timetic,cps;

  public spiel(main parent)
  { mama=parent;
    sxmax=mama.gamepanel.sxmax;
    timetic=mama.gamepanel.timetic;
    cps=1000/timetic;
    zaund=mama.my_sound;
    init_data();
  }

// -------------------------------------------------------------------------------------
// Slots:
// -------------------------------------------------------------------------------------
  public void set_pause(boolean pause)     { pausiert=pause; }
  public void set_sound(boolean mit_sound) { soundon=mit_sound; }
  public void set_messi(String t)          { messi=t; }
  public boolean get_sound()               { return soundon;  }
  public boolean get_paused()              { return pausiert; }

  public void neu(int lv,int pkt)
  { int i;
    level=lv; punkte=pkt; punkte0=punkte; gover=false; pausiert=false; level_fertig=false;
    for (i=0; i<maxsprites; i++) kugel[i].stoppen();
    for (i=0; i<maxfloater; i++) { floater[i].typ=-1; floater[i].stoppen(); }
    for (i=0; i<haufen.length; i++) haufen[i].stoppen();
    maxcount=calc_koo(level); // Level holen
    if (maxcount<1) { mama.popup("Sorry, das war das letzte Level!"); weitermit=4; }
    else
    { kliste[0]=anzahl;
      for (i=1; i<=kliste[0]; i++) { kliste[i]=i-1; kugel[kliste[i]].typ=get_kugeltyp(); } // Kette erstellen
      start.moveto(koo[0][0]-16,koo[0][1]-16,7);
      ziel.moveto(koo[maxcount-1][0]-16,koo[maxcount-1][1]-16,7);
      shooter.moveto(256,256,8);
      sync(0);
      weitermit=1;
      stopcount=0; // Tics, die die Schlange aussetzen soll
    } 
  }

  public void klick(int ix,int iy)
  { if (!pausiert) if (!gover) { if (soundon) zaund.play_sound(0); flyer.vy=-8; }
    messi="";
  }

  public void hover(int ix,int iy)
  { int mitte=ix;
    if (mitte<0) mitte=0; else if (mitte>sxmax) mitte=sxmax; // Hover-Bereichsgrenzen
    shooter.moveto(mitte-shooter.hotx,shooterline,8);
    if (flyer.vy==0) flyer.moveto(mitte-flyer.hotx,shooter.y+32,9);
  }

// -------------------------------------------------------------------------------------
// interne Methoden:
// -------------------------------------------------------------------------------------

  void init_data()
  { int i;
    level=0; punkte=0; gover=false; pausiert=false;
    for (i=0; i<maxsprites; i++) kugel[i]=new sprite(i,1,24,24);
    for (i=0; i<kliste.length; i++) kliste[i]=-1;
    shooter=new sprite(maxsprites,0,64,64);
    flyer=new sprite(maxsprites+1,0,24,24);
    start=new sprite(maxsprites+2,0,64,64);
    ziel =new sprite(maxsprites+3,0,64,64);
    for (i=0; i<maxhaufen; i++) haufen[i]=new sprite(maxsprites+4+i,0,64,64);
    for (i=0; i<maxfloater; i++) floater[i]=new sprite(maxsprites+4+maxhaufen+i,0,24,24);
    weitermit=20; // Titelbild zeigen
    mainloop=new Thread(this);
    mainloop.start();
  }

  public void run()
  { int i,temp=-1; // fuer Datenuebergabe zwischen Ani-ketten 
    while (mainloop!=null)
    { if (!gover) if (!pausiert) switch (weitermit)
      { case 0: temp=moveit(); // Schlange und Flyer bewegen, Kollision feststellen (temp:Platz in Kette oder 0)
                if (temp==0) weitermit=1;  // rausgeflogen, Kroete/Schlange oder Goodie getroffen
                else if (temp>0) { flyer.stoppen(); multi=0; weitermit=2; }  // Treffer auswerten
                else if (kugel[kliste[1]].countdown>=maxcount-1) weitermit=3;    // gameover
                break;
        case 1: init_flyer(); weitermit=0; break;
        case 2: get_gruppe(temp);
                if (dliste[0]<3) weitermit=1; else
                { multi++; punkte=punkte+multi*dliste[0]*dliste[0];
                  messi=mama.langdef.get("SCORE")+": "+dliste[0]*dliste[0]+"x"+multi;
                  weitermit=4;
                }
                break;
        case 3: gover=true;
                if (soundon) zaund.play_sound(5);
                mama.hsc.eintrag(punkte);
                mama.gamepanel.set_screen(1);
                weitermit=20;
                break;
        case 4: if (check_sonder()) for (i=0; i<3; i++) init_floater(); kugeln_explo(); weitermit++;  break;
        case 5: weitermit++; break;
        case 6: kugeln_explo2();  if (soundon) zaund.play_sound(6); weitermit++; break;   // Explo Phase 2
        case 7: weitermit++; break;
        case 8: kugeln_loeschen(); weitermit++; break; // Explo Phase 3
        case 9: weitermit++; break;
        case 10:temp=kette_zusammenziehen(); if (kliste[0]<1) weitermit=11; else weitermit=2; break;
        case 11: neu(level+1,punkte); break; // Levelup
        case 20: break;                      // Haltepunkt
      }
      try { Thread.sleep(timetic); }
      catch (InterruptedException e) { break; }
    }
  }
  public void stop()    { mainloop=null; }
  public void destroy() { stop(); }

// -------------------------------------------------------------------------------------------------------
// Sprites bewegen, Kollision feststellen, geschossene Goodies auswerten und bei Bedarf Flyer in Kette einhaengen
// return: Platz, den der Flyer in der Kette einnimmt, oder 0, um neuen Flyer zu generieren, sonst -1
  synchronized int moveit()
  { int i,k,coll=-1,colltyp=-1;
    if (stopcount>0) { sync(kugel[kliste[1]].countdown); stopcount--; } else sync(kugel[kliste[1]].countdown+1);
    for (i=0; i<maxfloater; i++) if (floater[i].vy!=0.0)
    { if (Math.random()<0.33) floater[i].vx=3.0*Math.random()-1.0;
      floater[i].move_it(); if (floater[i].y>shooterline+shooter.hoehe) { floater[i].typ=-1; floater[i].stoppen(); }
    }
    if (flyer.vy!=0.0) // flyermove und Collision Detection:
    { flyer.move_it();
      if (kugel_mit_rechteck(flyer,start)) coll=0;
      else if (kugel_mit_rechteck(flyer,ziel))  coll=0;
      else for (i=0; i<haufen.length; i++) if (kugel_mit_rechteck(flyer,haufen[i])) coll=0;
      if (coll<0) for (i=0; i<maxfloater; i++) if (floater[i].typ>0) if (kugel_mit_kugel(flyer,floater[i])>0) 
      { coll=0;
        floater_auswerten(floater[i].typ);
        floater[i].stoppen();
        floater[i].typ=-1;
      }
      if (coll==0) { if (soundon) zaund.play_sound(3); } else if (flyer.y<-flyer.hoehe) coll=0; // nach oben abgehauen
      if (coll<0) // Kugelkollision:
      { for (i=1; i<=kliste[0]; i++) if (kugel[kliste[i]].countdown>=0) // nur "aktive" Kugeln
        { k=kugel_mit_kugel(flyer,kugel[kliste[i]]);  // 1:vor Zentrum, 2:Volltreffer, 3:nach Zentrum
          if (k>0) 
          { if (coll<0) { coll=i; colltyp=k; }
            else if (kugel[kliste[i]].y>kugel[kliste[coll]].y) { coll=i; colltyp=k; } // unterste Kugel nehmen
          }
        }
        if (coll>=0) coll=einsetzen(coll,colltyp);
      }
    }
    return coll;
  }

  // Kugelphasen auf Kopfphase synchronisieren, direktes Ansprechen war schneller als via kugel.moveto
  void sync(int phase0) 
  { int i,j,k;
    j=phase0;
    for (i=1; i<=kliste[0]; i++)
    { k=kliste[i];
      if (j<0) kugel[k].stoppen(); else { kugel[k].countdown=j; kugel[k].moveto(koo[j][0],koo[j][1]-1,-1); }
      j=j-ktic;
    }
  }

// -----------------------------------------------------------------------------------------------------
// diverse Startups:
// -----------------------------------------------------------------------------------------------------
  void levelup() { level++; neu(level,punkte); }

  // levelabhaengig farbige Kugeln fuer die Kette generieren:
  int get_kugeltyp() 
  { int k=numfarben;
    k=(int)(k*Math.random())%k+1;
    if (joker_on)  if (Math.random()<0.02*numfarben) k=joker;
    if (sonder_on) if (Math.random()<0.02*numfarben) k=sonder;
    return k;
  }

  // Flyerkugel generieren, nur Farben verwenden, die in der Kette vorkommen
  int get_flyertyp()
  { int i,k=-1;
    int[] l={ 0,0,0,0,0,0,0,0 };
    if (kliste[0]>0)
    { for (i=1; i<=kliste[0]; i++) l[kugel[kliste[i]].typ]++; // Farbhaeufigkeit
      while (k<0) { k=(int)(8*Math.random())%8; if (l[k]<1) k=-1; }
    }
    else  k=joker; // irgendwas draufpappen, Hauptsache kompatibel
    return k;
  }

  // Shooter laden:
  void init_flyer()
  { flyer.typ=get_flyertyp();
    flyer.vy=0;
    hover(shooter.x+shooter.hotx,shooter.y+shooter.hoty);
  }

 // --------------------------------------------------------------------------------------------------------
 // Auswertung und Animationen: 
 // --------------------------------------------------------------------------------------------------------
   // Kugel je nach Treffertyp einordnen, neuen Platz zurueckgeben:
   int einsetzen(int platz,int colltyp)
   { int i,k=kugel[kliste[platz]].countdown,pneu=platz,kopf=kugel[kliste[1]].countdown;
     if (colltyp==2) { if (Math.random()<0.5) colltyp=3; else colltyp=1; } // irgendwo muss die Kugel zwischen
     if (koo[k+1][0]>koo[k][0]) { if (colltyp==1) pneu=platz+1; else pneu=platz; } // Kette laeuft nach rechts:
     else if (koo[k+1][0]<koo[k][0]) { if (colltyp==1) pneu=platz; else pneu=platz+1; } // Kette laeuft nach links

     if (pneu<1) pneu=1; if (pneu>kliste[0]) pneu=kliste[0]+1;         // Randeffekte abfangen
     k=-1; for (i=0; i<maxsprites; i++) if (in_liste(i,kliste)<1) k=i; // nicht benutzte Kugel  suchen
     if (k<0) { k=kliste[kliste[0]]; kliste[0]--; }          // Kugel vom Kettenende "ausleihen"
     kugel[k].typ=flyer.typ;                                 // Flyer-Typ uebergeben
     kliste[0]++; for (i=kliste[0]; i>pneu; i--) kliste[i]=kliste[i-1]; kliste[pneu]=k; // in Kugelliste einfuegen
     sync(kopf+ktic);
     if (soundon) zaund.play_sound(1); 
     return pneu;
   }

   // Triplets oder hoeheres an Trefferstelle, ujas Flood-Algorithmus:
   void get_gruppe(int treffer)
   { int i,k,p1,p2;
     boolean weiter=true;
     dliste[0]=1; dliste[1]=treffer;
     p1=0;
     while (weiter)
     { p2=dliste[0];
       for (i=p1+1; i<=p2; i++)
       { k=dliste[i];
         if (k>1) if (passt(kugel[kliste[k]].typ,kugel[kliste[k-1]].typ)) add_liste(k-1,dliste);
         if (k<kliste[0]) if (passt(kugel[kliste[k]].typ,kugel[kliste[k+1]].typ)) add_liste(k+1,dliste);
       }
       weiter=(p2<dliste[0]);
       if (weiter) p1=p2;
     }
   }
   boolean passt(int n1,int n2)
   { boolean ok=((n1==joker)||(n2==joker));
     if (!ok) ok=(n1==n2);
     return ok;
   }

   // Kugelexplosion-Ani:
   void kugeln_explo()
   { if (dliste[0]>0) for (int i=1; i<=dliste[0]; i++) kugel[kliste[dliste[i]]].typ=explo;
     if (soundon) zaund.play_sound(6);
   }
   void kugeln_explo2() { if (dliste[0]>0) for (int i=1; i<=dliste[0]; i++) kugel[kliste[dliste[i]]].typ++; }
   void kugeln_loeschen() { if (dliste[0]>0) for (int i=1; i<=dliste[0]; i++) kugel[kliste[dliste[i]]].typ=weg; }

   // Kugelliste nach hinten zusammenziehen:
   synchronized int kette_zusammenziehen()
   { int i,j,k=0,min=-1,kopf=kugel[kliste[1]].countdown;
     for (i=1; i<=kliste[0]; i++)
     { j=kliste[i];
       if (kugel[j].typ==weg) { kopf=kopf-ktic; if (min<0) min=i; } else { k++; kliste[k]=j; }
     }
     kliste[0]=k;
     if (k>0) { sync(kopf); if (min>1) if (soundon) zaund.play_sound(2); } // heftiges Knallen
     else { level_fertig=true; if (soundon) zaund.play_sound(4); }         // kurzes Zischen, da nur Kopf weg
     return min;
   }

// ===================================================================================================
// Level aufbereiten:
// ---------------------------------------------------------------------------------------------------
  // Kodierung: 1.Zeile: Start koordinaten, 2. und weitere: Himmelsrichtung(16er Windrose), Anz.Steps
  int calc_koo(int lev)
  { final double size=1.0,sin45=0.707106781,sin22p5=0.382683432,sin67p5=0.923879532; // fuer 16 Winkel
    int i,j,k,hcnt=0,cnt=0;
    double dx=0,dy=0;
    double kood[][]=new double[maxstep][2];
    String[] ko=new String[3];
    String stationen[]=mama.fh.get_ressourcelines("/"+mama.datenpfad+"level."+lev);
    joker_on=false;
    sonder_on=false;
    numfarben=3;
    anzahl=32;
    k=0;
    if (stationen[0]!="") for (i=0; i<stationen.length; i++)
    { ko=stationen[i].split(",");
      if (ko[0].equals("start")) 
      { kood[k][0]=Double.parseDouble(ko[1]);
        kood[k][1]=Double.parseDouble(ko[2]);
      }
      else if (ko[0].equals("block"))
      { haufen[hcnt].set_typ(1);
        haufen[hcnt].starten();
        haufen[hcnt].moveto(Integer.parseInt(ko[1]),Integer.parseInt(ko[2]),9);
        hcnt++;
      }
      else if (ko[0].equals("farben")) numfarben=Integer.parseInt(ko[1]);
      else if (ko[0].equals("kugeln")) anzahl=Integer.parseInt(ko[1]);
      else if (ko[0].equals("joker"))  joker_on=true;
      else if (ko[0].equals("sonder")) sonder_on=true; // neu in V.1.1
      else
      { cnt=2*Integer.parseInt(ko[1]);
        if (ko[0].equals("N")) { dx=0; dy=-size; }
        else if (ko[0].equals("NNO")) { dx=size*sin22p5; dy=-size*sin67p5; }
        else if (ko[0].equals("NO"))  { dx=size*sin45;   dy=-size*sin45; }
        else if (ko[0].equals("ONO")) { dx=size*sin67p5; dy=-size*sin22p5; }
        else if (ko[0].equals("O"))   { dx=size; dy=0; }
        else if (ko[0].equals("OSO")) { dx=size*sin67p5;  dy=size*sin22p5; }
        else if (ko[0].equals("SO"))  { dx=size*sin45;    dy=size*sin45; }
        else if (ko[0].equals("SSO")) { dx=size*sin22p5;  dy=size*sin67p5; }
        else if (ko[0].equals("S"))   { dx=0; dy=size; }
        else if (ko[0].equals("SSW")) { dx=-size*sin22p5; dy=size*sin67p5; }
        else if (ko[0].equals("SW"))  { dx=-size*sin45;   dy=size*sin45; }
        else if (ko[0].equals("WSW")) { dx=-size*sin67p5; dy=size*sin22p5; }
        else if (ko[0].equals("W"))   { dx=-size; dy=0; }
        else if (ko[0].equals("WNW")) { dx=-size*sin67p5; dy=-size*sin22p5; }
        else if (ko[0].equals("NW"))  { dx=-size*sin45;   dy=-size*sin45; }
        else if (ko[0].equals("NNW"))  { dx=-size*sin22p5; dy=-size*sin67p5; }
        else System.out.println("Uknown direction in level "+lev+", line "+i);
        for (j=1; j<cnt; j++) if (k<maxstep-1)
        { kood[k+1][0]=kood[k][0]+dx; kood[k+1][1]=kood[k][1]+dy;
          k++;
        }
      }
    }
    // Feinkoordinaten zu Grobkoordinaten (Integers)
    if (k>0) for (i=0; i<k; i++) { koo[i][0]=(int)(kood[i][0]); koo[i][1]=(int)(kood[i][1]); }
    return k;
  }

// ======================================================================================================
// Listenhelferlein:
// ---------------------------------------------------------------------------------------------------
   int in_liste(int nr,int[]l) { int i,k=0; for (i=1; i<=l[0]; i++) if (l[i]==nr) k=i; return k; }
   void add_liste(int nr,int[]l) { if (in_liste(nr,l)<1)  { l[0]++; ; l[l[0]]=nr; } }

// ======================================================================================================
// Crash Detection:
  // Kugel mit Schlangenkopf, Mittelpunktcrash sah am besten aus:
  boolean kugel_mit_rechteck(sprite ku,sprite eck) { return eck.in_rechteck(ku.x+ku.hotx,ku.y+ku.hoty); }

  // 2 Kugeln, Pythagoras: 0: kein Crash, 1: Mitte1x<Mitte2x, 2: Mitte1x==Mitte2x, 3: Mitte1x>Mitte2x
  int kugel_mit_kugel(sprite ku1,sprite ku2)
  { int crash=0,ix=ku1.x-ku2.x,iy=ku1.y-ku2.y,iq=ku1.hotx+ku2.hotx;
    if (ix*ix+iy*iy<=iq*iq) { if (ix<0) crash=1; else if (ix>0) crash=3; else crash=2; }
    return crash;
  }
// ======================================================================================================
// Version 1.1:
// Floater von y=0 irgendwie nach unten, dann abschalten
// wird generiert bei Crash brauner Kugeln, muss abgeschossen werden
// Das Java-System scheint hiermit am Ende, obwohl die Methoden nur bei Crash angesprungen werden
// (zu viele Sprites und Crash-Detections?)
// bisherige Sondersteine: Kette vor, Kette zurueck, +50P, -50P
// -------------------------------------------------------------------------------------------------------
   void init_floater()
   { int i,k=-1;
     for (i=0; i<maxfloater; i++) if (k<0) if (floater[i].typ<1) k=i;
     floater[k].moveto((int)((sxmax-floater[k].breite)*Math.random()%(sxmax-floater[k].breite)),0,-1);
     floater[k].vx=0; floater[k].vy=1;
     i=(int)(5*Math.random())%5+1;
     floater[k].typ=i;
   }

   void floater_auswerten(int nr)
   { switch (nr)
     { case 1: if (kliste[0]>0) kugel[kliste[1]].countdown=kugel[kliste[1]].countdown+72; break; // 3 vorschieben
       case 2: if (kliste[0]>0) kugel[kliste[1]].countdown=kugel[kliste[1]].countdown-72; break; // 3 zurueckziehen
       case 3: punkte=punkte+50; break;
       case 4: punkte=punkte-50; break;
       default: stopcount=5*cps; break; // 5s paralysieren
     }
   }

  boolean check_sonder()
  { boolean ist_sonder=false;
    if (dliste[0]>0) for (int i=1; i<=dliste[0]; i++) if (kugel[kliste[dliste[i]]].typ==sonder) ist_sonder=true;
    return ist_sonder;
  }
// ----------------------------------------------------


}
